CodeQL / Analyze (actions) (push) Canceled after 0s
CodeQL / Analyze (python) (push) Canceled after 0s
pre-commit / pre-commit (push) Canceled after 0s
Multi-Arch CI / setup (push) Canceled after 0s
Publish build_manylinux_x86_64 images / publish_build_manylinux_x86_64 (push) Canceled after 0s
Setup (Multi-Arch) / setup (push) Canceled after 0s
Multi-Arch CI / Linux::${{ fromJSON(needs.setup.outputs.linux_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubi10 image / publish_no_rocm_image_ubi10 (push) Canceled after 0s
Multi-Arch CI / Windows::${{ fromJSON(needs.setup.outputs.windows_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Multi-Arch CI / Manifest Diff (push) Canceled after 0s
TheRock Manifest Diff Report / Generate Manifest Diff Report (push) Canceled after 0s
Multi-Arch CI / CI Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_ocl_rt images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_rocgdb images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Unit Tests / Unit Tests :: ubuntu-24.04 (push) Canceled after 0s
Unit Tests / Unit Tests :: windows-2022 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_media (push) Canceled after 0s
Unit Tests / Unit Tests Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_openmpi (push) Canceled after 0s
123 lines
3.6 KiB
C++
123 lines
3.6 KiB
C++
/*
|
|
* Copyright (c) Advanced Micro Devices, Inc., or its affiliates.
|
|
*
|
|
* SPDX-License-Identifier: MIT
|
|
*/
|
|
|
|
#include <iostream>
|
|
#include <hip/hip_runtime.h>
|
|
#include "hip_helper.h"
|
|
|
|
#define WIDTH 32
|
|
|
|
#define NUM (WIDTH * WIDTH)
|
|
|
|
#define THREADS_PER_BLOCK_X 4
|
|
#define THREADS_PER_BLOCK_Y 4
|
|
#define THREADS_PER_BLOCK_Z 1
|
|
|
|
using namespace std;
|
|
|
|
__global__ void matrixTranspose_static_shared(float* out, float* in, const int width) {
|
|
__shared__ float sharedMem[WIDTH * WIDTH];
|
|
|
|
int x = blockDim.x * blockIdx.x + threadIdx.x;
|
|
int y = blockDim.y * blockIdx.y + threadIdx.y;
|
|
|
|
sharedMem[y * width + x] = in[x * width + y];
|
|
|
|
__syncthreads();
|
|
|
|
out[y * width + x] = sharedMem[y * width + x];
|
|
}
|
|
|
|
__global__ void matrixTranspose_dynamic_shared(float* out, float* in, const int width) {
|
|
extern __shared__ float sharedMem[];
|
|
|
|
int x = blockDim.x * blockIdx.x + threadIdx.x;
|
|
int y = blockDim.y * blockIdx.y + threadIdx.y;
|
|
|
|
sharedMem[y * width + x] = in[x * width + y];
|
|
|
|
__syncthreads();
|
|
|
|
out[y * width + x] = sharedMem[y * width + x];
|
|
}
|
|
|
|
void MultipleStream(float** data, float* randArray, float** gpuTransposeMatrix,
|
|
float** TransposeMatrix, int width) {
|
|
const int num_streams = 2;
|
|
hipStream_t streams[num_streams];
|
|
|
|
for (int i = 0; i < num_streams; i++) checkHipErrors(hipStreamCreate(&streams[i]));
|
|
|
|
for (int i = 0; i < num_streams; i++) {
|
|
checkHipErrors(hipMalloc((void**)&data[i], NUM * sizeof(float)));
|
|
checkHipErrors(
|
|
hipMemcpyAsync(data[i], randArray, NUM * sizeof(float), hipMemcpyHostToDevice, streams[i]));
|
|
}
|
|
|
|
hipLaunchKernelGGL(matrixTranspose_static_shared,
|
|
dim3(WIDTH / THREADS_PER_BLOCK_X, WIDTH / THREADS_PER_BLOCK_Y),
|
|
dim3(THREADS_PER_BLOCK_X, THREADS_PER_BLOCK_Y), 0, streams[0],
|
|
gpuTransposeMatrix[0], data[0], width);
|
|
|
|
hipLaunchKernelGGL(matrixTranspose_dynamic_shared,
|
|
dim3(WIDTH / THREADS_PER_BLOCK_X, WIDTH / THREADS_PER_BLOCK_Y),
|
|
dim3(THREADS_PER_BLOCK_X, THREADS_PER_BLOCK_Y), sizeof(float) * WIDTH * WIDTH,
|
|
streams[1], gpuTransposeMatrix[1], data[1], width);
|
|
|
|
for (int i = 0; i < num_streams; i++)
|
|
checkHipErrors(hipMemcpyAsync(TransposeMatrix[i], gpuTransposeMatrix[i], NUM * sizeof(float),
|
|
hipMemcpyDeviceToHost, streams[i]));
|
|
}
|
|
|
|
int main() {
|
|
checkHipErrors(hipSetDevice(0));
|
|
|
|
float *data[2], *TransposeMatrix[2], *gpuTransposeMatrix[2], *randArray;
|
|
|
|
int width = WIDTH;
|
|
|
|
randArray = (float*)malloc(NUM * sizeof(float));
|
|
|
|
TransposeMatrix[0] = (float*)malloc(NUM * sizeof(float));
|
|
TransposeMatrix[1] = (float*)malloc(NUM * sizeof(float));
|
|
|
|
checkHipErrors(hipMalloc((void**)&gpuTransposeMatrix[0], NUM * sizeof(float)));
|
|
checkHipErrors(hipMalloc((void**)&gpuTransposeMatrix[1], NUM * sizeof(float)));
|
|
|
|
for (int i = 0; i < NUM; i++) {
|
|
randArray[i] = (float)i * 1.0f;
|
|
}
|
|
|
|
MultipleStream(data, randArray, gpuTransposeMatrix, TransposeMatrix, width);
|
|
|
|
checkHipErrors(hipDeviceSynchronize());
|
|
|
|
// verify the results
|
|
int errors = 0;
|
|
double eps = 1.0E-6;
|
|
for (int i = 0; i < NUM; i++) {
|
|
if (std::abs(TransposeMatrix[0][i] - TransposeMatrix[1][i]) > eps) {
|
|
printf("%d stream0: %f stream1 %f\n", i, TransposeMatrix[0][i], TransposeMatrix[1][i]);
|
|
errors++;
|
|
}
|
|
}
|
|
if (errors != 0) {
|
|
printf("FAILED: %d errors\n", errors);
|
|
} else {
|
|
printf("stream PASSED!\n");
|
|
}
|
|
|
|
free(randArray);
|
|
for (int i = 0; i < 2; i++) {
|
|
checkHipErrors(hipFree(data[i]));
|
|
checkHipErrors(hipFree(gpuTransposeMatrix[i]));
|
|
free(TransposeMatrix[i]);
|
|
}
|
|
|
|
checkHipErrors(hipDeviceReset());
|
|
return 0;
|
|
}
|